@@ -1,11 +1,16 @@
-################################################################################
-#
-# Copyright (c) 2019, the Perspective Authors.
-#
-# This file is part of the Perspective library, distributed under the terms of
-# the Apache License 2.0.  The full license can be found in the LICENSE file.
-#
-import tornado
+#  ┏━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┓
+#  ┃ ██████ ██████ ██████       █      █      █      █      █ █▄  ▀███ █       ┃
+#  ┃ ▄▄▄▄▄█ █▄▄▄▄▄ ▄▄▄▄▄█  ▀▀▀▀▀█▀▀▀▀▀ █ ▀▀▀▀▀█ ████████▌▐███ ███▄  ▀█ █ ▀▀▀▀▀ ┃
+#  ┃ █▀▀▀▀▀ █▀▀▀▀▀ █▀██▀▀ ▄▄▄▄▄ █ ▄▄▄▄▄█ ▄▄▄▄▄█ ████████▌▐███ █████▄   █ ▄▄▄▄▄ ┃
+#  ┃ █      ██████ █  ▀█▄       █ ██████      █      ███▌▐███ ███████▄ █       ┃
+#  ┣━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┫
+#  ┃ Copyright (c) 2017, the Perspective Authors.                              ┃
+#  ┃ ╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌ ┃
+#  ┃ This file is part of the Perspective library, distributed under the terms ┃
+#  ┃ of the [Apache License 2.0](https://www.apache.org/licenses/LICENSE-2.0). ┃
+#  ┗━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━━┛
+
+import asyncio
 
 from random import random
 from functools import partial
@@ -47,7 +52,7 @@ def view(
         "config": config,
     }
 
-    future = tornado.concurrent.Future()
+    future = asyncio.Future()
     client.post(msg, future)
     return future
 
@@ -108,9 +113,7 @@ class PerspectiveViewProxy(object):
         return self._async_queue("schema", "view_method", as_string=as_string)
 
     def expression_schema(self, as_string=False):
-        return self._async_queue(
-            "expression_schema", "view_method", as_string=as_string
-        )
+        return self._async_queue("expression_schema", "view_method", as_string=as_string)
 
     def on_update(self, callback, mode=None):
         return self._subscribe("on_update", "view_method", callback, mode=mode)
@@ -150,3 +153,6 @@ class PerspectiveViewProxy(object):
 
     def to_columns(self, **kwargs):
         return self._async_queue("to_columns", "view_method", **kwargs)
+
+    def to_columns_string(self, **kwargs):
+        return self._async_queue("to_columns_string", "view_method", **kwargs)
